/*************************************************************************************************
Dmitri Korotkevitch
24HOP - Data partitioning in VLDB (second edition)

http://aboutsqlserver.com
email: dmitri@aboutsqlserver.com

Table creation
(SQL Server 2012 - Enterprise Edition required)
*************************************************************************************************/

use PartitioningDb
go

-- 2011 Data
create partition function pfOrders2011(datetime2(0))
as range right for values
(
	'2011-02-01','2011-03-01','2011-04-01','2011-05-01'
	,'2011-06-01','2011-07-01','2011-08-01','2011-09-01'
	,'2011-10-01','2011-11-01','2011-12-01'
)
go

create partition scheme psOrders2011
as partition pfOrders2011
all to ([Data2011])
go

create table dbo.Orders2011
(
	OrderId int not null,
	OrderDate datetime2(0) not null,
	OrderNum varchar(32) not null,
	Placeholder char(100) null,
	
	constraint CHK_Orders2011_Final
	check(OrderDate >= '2011-01-01' and OrderDate < '2012-01-01')
)
on[Data2011]
go

create unique clustered index IDX_Orders2011_OrderDate_OrderID
on dbo.Orders2011(OrderDate,OrderId)
with (data_compression=page)
on psOrders2011(OrderDate)
go

create unique nonclustered index IDX_Orders2011_OrderNum_OrderDate
on dbo.Orders2011(OrderNum,OrderDate)
with (data_compression=page)
on psOrders2011(OrderDate)
go


-- 2012 main table
create partition function pfOrders2012(datetime2(0))
as range right for values
(
	'2012-02-01','2012-03-01','2012-04-01','2012-05-01'
	,'2012-06-01','2012-07-01','2012-08-01','2012-09-01'
	,'2012-10-01','2012-11-01','2012-12-01'
)
go

create partition scheme psOrders2012
as partition pfOrders2012
all to ([Data2012])
go

create table dbo.Orders2012
(
	OrderId int not null,
	OrderDate datetime2(0) not null,
	OrderNum varchar(32) not null,
	Placeholder char(100) null,
	
	constraint CHK_Orders2012
	check(OrderDate >= '2012-01-01' and OrderDate < '2012-12-01')
)
on [Data2012]
go

create unique clustered index IDX_Orders2012_OrderDate_OrderID
on dbo.Orders2012(OrderDate,OrderId)
with (data_compression=page)
on psOrders2012(OrderDate)
go

create unique nonclustered index IDX_Orders2012_OrderNum_OrderDate
on dbo.Orders2012(OrderNum,OrderDate)
with (data_compression=page)
on psOrders2012(OrderDate)
go

-- 2012 December
create table dbo.Orders2012_12
(
	OrderId int not null,
	OrderDate datetime2(0) not null,
	OrderNum varchar(32) not null,
	Placeholder char(100) null,
	
	constraint CHK_Orders2012_12
	check(OrderDate >= '2012-12-01' and OrderDate < '2013-01-01')
)
on [FASTSTORAGE]
go

create unique clustered index IDX_Orders2012_12_OrderDate_OrderID
on dbo.Orders2012_12(OrderDate,OrderId)
on [FASTSTORAGE]
go

create unique nonclustered index IDX_Orders2012_12_OrderNum_OrderDate
on dbo.Orders2012_12(OrderNum,OrderDate)
on [FASTSTORAGE]
go


-- View
create view dbo.Orders(OrderDate, OrderId, OrderNum, Placeholder)
as
	select OrderDate, OrderId, OrderNum, PlaceHolder
	from dbo.Orders2012_12

	union all 

	select OrderDate, OrderId, OrderNum, PlaceHolder
	from dbo.Orders2012

	union all 

	select OrderDate, OrderId, OrderNum, PlaceHolder
	from dbo.Orders2011
go

-- Populating data 2011
;with N1(C) as (select 0 union all select 0) -- 2 rows
,N2(C) as (select 0 from N1 as T1 cross join N1 as T2) -- 4 rows
,N3(C) as (select 0 from N2 as T1 cross join N2 as T2) -- 16 rows
,N4(C) as (select 0 from N3 as T1 cross join N3 as T2) -- 256 rows
,N5(C) as (select 0 from N4 as T1 cross join N4 as T2) -- 65,536 rows
,IDs(ID) as (select ROW_NUMBER() over (order by (select NULL)) from N5)
insert into dbo.Orders2011(OrderDate,OrderId,OrderNum)
	select  
		DATEADD(hour,ID % (365 * 24),'2011-01-01'),
		ID,
		'Order: ' + convert(varchar(12),ID)	
	from IDs
go

-- populating 2012 (Jan-Nov)
;with N1(C) as (select 0 union all select 0) -- 2 rows
,N2(C) as (select 0 from N1 as T1 cross join N1 as T2) -- 4 rows
,N3(C) as (select 0 from N2 as T1 cross join N2 as T2) -- 16 rows
,N4(C) as (select 0 from N3 as T1 cross join N3 as T2) -- 256 rows
,N5(C) as (select 0 from N4 as T1 cross join N4 as T2) -- 65,536 rows
,IDs(ID) as (select 100000 + ROW_NUMBER() over (order by (select NULL)) from N5)
insert into dbo.Orders2012(OrderDate,OrderId,OrderNum)
	select  
		DATEADD(hour,ID % (330 * 24),'2012-01-01'),
		ID,
		'Order: ' + convert(varchar(12),ID)	
	from IDs
go

-- Populating Dec 2012
;with N1(C) as (select 0 union all select 0) -- 2 rows
,N2(C) as (select 0 from N1 as T1 cross join N1 as T2) -- 4 rows
,N3(C) as (select 0 from N2 as T1 cross join N2 as T2) -- 16 rows
,N4(C) as (select 0 from N3 as T1 cross join N3 as T2) -- 256 rows
,N5(C) as (select 0 from N4 as T1 cross join N3 as T2) -- 4096 rows
,IDs(ID) as (select 200000 + ROW_NUMBER() over (order by (select NULL)) from N5)
insert into dbo.Orders2012_12(OrderDate,OrderId,OrderNum)
	select  
		DATEADD(hour,ID % (31 * 24),'2012-12-01'),
		ID,
		'Order: ' + convert(varchar(12),ID)	
	from IDs
go

