/*************************************************************************************************
Dmitri Korotkevitch
24HOP - Data partitioning in VLDB (second edition)

http://aboutsqlserver.com
email: dmitri@aboutsqlserver.com

Regular table - creation script
(SQL Server 2012 - Enterprise Edition required)
*************************************************************************************************/


set nocount on
set xact_abort on
go

use Partitioning2012
go

if exists
(
	select * 
	from sys.tables t join sys.schemas s on t.schema_id = s.schema_id
	where s.name = 'dbo' and t.name = 'Data2'
)
	drop table dbo.Data2
go


if exists
(
	select * 
	from sys.partition_schemes
	where name = 'psData2'
)
	drop partition scheme psData2
go

if exists
(
	select * 
	from sys.partition_functions
	where name = 'pfData2'
)
	drop partition function pfData2
go

create table dbo.Data2
(
	OrderDate date not null,
	OrderId int not null identity(1,1),
	OrderNum varchar(32) not null,
	LobColumn varchar(max) null,
	Placeholder char(50) null,
)
TEXTIMAGE_ON [FG1]
go

create unique clustered index IDX_Data2_OrderDate_OrderId
on dbo.Data2(OrderDate, OrderId)
on [FG1]
go

declare
	@StartDate datetime = '2012-12-01'

;with N1(C) as (select 0 union all select 0) -- 2 rows
,N2(C) as (select 0 from N1 as T1 cross join N1 as T2) -- 4 rows
,N3(C) as (select 0 from N2 as T1 cross join N2 as T2) -- 16 rows
,N4(C) as (select 0 from N3 as T1 cross join N3 as T2) -- 256 rows
,IDs(ID) as (select ROW_NUMBER() over (order by (select NULL)) from N4)
insert into dbo.Data2(OrderDate, OrderNum)
	select 
		dateadd(day,[days].ID,@StartDate), 
		'Order: ' + convert(varchar(12),dateadd(day,[days].ID,@StartDate)) + '-' + convert(varchar(5),[orders].Id)
	from IDs [days] cross join IDs [orders]
go

select count(*) from dbo.Data2
go
